<?php

namespace Neuron\Person\Medical\Storage;

class Mysql extends \Neuron\Db\Storage implements Skeleton {

	protected $_nafManager;

	public function listdata($params) {
		$select = $this->select()->from(array('med' => $this->_tables['person_medical']));
		$select->join(array('par' => $this->_tables['person_parameters']), 'par.param_ref_id = med.medical_type_id',array('medical_type_name' => 'value1'));
		$select->where(array('par.param_ref_name' => 'MEDICAL_TYPE'));

		if ((!empty($params['person_id']))) {
			$select->where(array('med.person_id' => $params['person_id']));
		}

		if ((!empty($params['medical_type']))) {
			switch ($params['medical_type']) {
				case 'SURGERIES':
					$select->where(array('med.medical_type_id' => 2));
					break;

				case 'ALLERGIES':
					$select->where(array('med.medical_type_id' => 3));
					break;

				case 'HEREDITARY':
					$select->where(array('med.medical_type_id' => 4));
					break;
				
				default:
					$select->where(array('med.medical_type_id' => 1));
					break;
			}
		}
		if(isset($params['order'][0]['column']) && !empty($params['order'][0]['column'])){
			$i = $params['order'][0]['column'];
			$dir = $params['order'][0]['dir'];
			$select->order(array($params['columns'][$i]['data'] . " " . $dir));
		}

		if(isset($params['length']) && !empty($post['length'])){
			$select->limit(intval($params['length']));
			$select->offset(intval($params['start']));
		}

        $data = $this->fetchAll($select);
        $dataAll = [];

        if($data) {
            foreach($data as $row) {
                array_push($dataAll, $row);
            }
        }

        $selectC = $this->select()->from(array('med' => $this->_tables['person_medical']));
	    $selectC->join(array('par' => $this->_tables['person_parameters']), 'par.param_ref_id = med.medical_type_id',array('medical_type_name' => 'value1'));
		$selectC->where(array('par.param_ref_name' => 'MEDICAL_TYPE'));
        $selectC->columns(['total_count' => new \Zend\Db\Sql\Expression('COUNT(*)')]);
        if ((!empty($params['person_id']))) {
			$selectC->where(array('med.person_id' => $params['person_id']));
		}

		if ((!empty($params['medical_type']))) {
			switch ($params['medical_type']) {
				case 'SURGERIES':
					$selectC->where(array('med.medical_type_id' => 2));
					break;

				case 'ALLERGIES':
					$selectC->where(array('med.medical_type_id' => 3));
					break;

				case 'HEREDITARY':
					$selectC->where(array('med.medical_type_id' => 4));
					break;
				
				default:
					$selectC->where(array('med.medical_type_id' => 1));
					break;
			}
		}
        $dataCount = $this->fetchRow($selectC);

        $total = ($dataCount['total_count'] > 0) ? $dataCount['total_count']:0;
                        
        $result = ['data' => $dataAll, 'total' => $total];
        
        return $result;
	}

	public function load($perMedId) {
		$select = $this->select()
						->from($this->_tables['person_medical'])
						->where(array('medical_id' => $perMedId));

		return $this->fetchRow($select);
	}
	
	public function save(\Neuron\Person\Medical $perMed, $update = false) {
		if ($update) {
			
			$update = $this->update()
				->table($this->_tables['person_medical'])
				->set($this->__($perMed->pull()))
				->where($this->__(array('medical_id' => $perMed->medical_id)));

			// echo str_replace('"','',$update->getSqlString()).'<br />';
			if ($this->execute($update)) {
					return $perMed->medical_id;
			}
			
		} else {

			$insert = $this->insert()
							->into($this->_tables['person_medical'])
							->autoincrement($this->_('medical_id'))
							->values($this->__($perMed->pull()));
			$save = $this->execute($insert);
			if ($save) {
					$perMed->medical_id = $save;
					return $perMed->medical_id;
			}
		}

		return false;
	}

	public function remove($perMedId) {
		$delete = $this->delete()
						->from($this->_tables['person_medical'])
						->where(array('medical_id' => $perMedId));
								
		return $this->execute($delete);
	}

	public function loadList($ids, $mode = \Neuron\Person\Medical\Storage::LIST_ALL, $params = null) {
        /* base select */
        $select = $this->select()
                ->from(array('a' => $this->_tables['person_medical']));

        /* loadby page */
        if ($mode == \Neuron\Person\Medical\Storage::LISTBY_PAGE) {
			if(isset($params['person_id']) && !empty($params['person_id'])){
				$select->where(array($this->_('person_id', 'a.') => $params['person_id']));
			}

            /* order */
            if(array_key_exists('order', $params) && array_key_exists('columns', $params)){
                foreach($params['order'] as $order){
                    $columnName = $params['columns'][$order['column']]['data'];
                    if($columnName == 'user_status_name'){
                        $colIdentity = $this->_('name', 'b.');
                    } else {
                        $colIdentity = $this->_($columnName, 'a.');
                    }
                    $select->order($colIdentity . ' ' . $order['dir']);
                }
            }
            
            /* get pager */
            $pager = \Neuron\Db\Storage\Pager::sanitize($params);
            $select->limit($pager->getLimit())->offset($pager->getOffset())->cache(0);

            /* has keyword? */
            if (!empty($ids)) {
                $select->where->nest()
                    ->like($this->_('company_name', 'd.'), '%' . trim($ids) . '%')
                    ->or
                    ->like($this->_('position', 'd.'), '%' . trim($ids) . '%')
					->or
                    ->like($this->_('job_desc', 'd.'), '%' . trim($ids) . '%')
                ->unnest();
            }

        } else {

            $pager = new \Neuron\Db\Storage\Pager(1, 999999);

            switch ($mode) {
                case \Neuron\Person\Medical\Storage::LISTBY_PERSON_TYPE:
                    $select->where(array($this->_('person_id', 'a.') => $ids));
                    break;
                case \Neuron\Person\Medical\Storage::LISTBY_TYPE:
                    $select->where(array($this->_('education_type', 'a.') => $ids));
                    break;
                default:
            }

        }

        $list = is_array($ids) ? $ids : ['id' => $ids];
        $list = is_array($params) ? array_merge($list, $params) : $list;
        $this->triggerQueryPerson($select, $list, __FUNCTION__);

        // echo str_replace('"','',$select->getSqlString()); die();
        try {
            if($rows = $this->fetchAll($select)){
                $result = new \stdClass();
                $result->list = $rows;
                $result->pager = $pager;
                $result->countTotal = $rows->total;
                return $result;
            }
            return false;
        } catch (\Exception $ex) {
            error_log('Error at ' . $ex->getFile() . ' line ' . $ex->getLine() . ': ' . $ex->getMessage());
            return false;
        }
    }

    private function triggerQueryPerson($query, $params, $caller) {
		
        /* trigger before query */
		if ($this->_nafManager = \Neuron\Application\Framework\Event\Manager::get()) {
			$this->_nafManager->events()->trigger('person_medical_query', $query, $params, $caller);
		}
        
    }

}